/*************************************************************************/
/* libsurl - library for generating shortened URLs                       */
/* Copyright (C) 2009  Scott Lawrence <bytbox@gmail.com> et al.          */
/*                                                                       */
/* This program is free software: you can redistribute it and/or modify  */
/* it under the terms of the GNU General Public License as published by  */
/* the Free Software Foundation, either version 3 of the License, or (at */
/* your option) any later version.                                       */
/*                                                                       */
/* This program is distributed in the hope that it will be useful, but   */
/* WITHOUT ANY WARRANTY; without even the implied warranty of            */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     */
/* General Public License for more details.                              */
/*                                                                       */
/* You should have received a copy of the GNU General Public License     */
/* along with this program.  If not, see <http://www.gnu.org/licenses/>. */
/*************************************************************************/

/* libsurl-fe.h - libsurl's frontend */

#ifndef LIBSURL_FE_H
#define LIBSURL_FE_H

#include <sys/wait.h>
#include <stdlib.h>
#include <unistd.h>
#include <pthread.h>


/*********************/
/* Preprocessor defs */
/*********************/

#define SURLOPT_USERNAME            0
#define SURLOPT_PASSWORD            1
#define SURLOPT_API_KEY             2
#define SURLOPT_TITLE               3
#define SURLOPT_SERVICE             4
#define SURLOPT_URL                 5
#define SURLOPT_SHORT_URL           6
#define SURLOPT_COUNT               7

#define SURLERR_NONE                0
#define SURLERR_USERNAME            1
#define SURLERR_PASSWORD            2
#define SURLERR_API_KEY             3
#define SURLERR_TITLE               4
#define SURLERR_SERVICE             5
#define SURLERR_URL                 6

#define SURL_MAX_SERVICES           500
#define SURL_CURL_MAX_BUFFER_SIZE   20000

#define A_GD                "a.gd"
#define A_NF                "a.nf"
#define BIT_LY              "bit.ly"
#define BURNURL             "burnurl.com"
#define CHILP_IT            "chilp.it"
#define CLI_GS              "cli.gs"
#define DECENTURL           "decenturl.com"
#define DIGG                "digg.com"
#define IS_GD               "is.gd"
#define KL_AM               "kl.am"
#define KURL_IT             "kurl.it"
#define LIIP_TO             "liip.to"
#define NOTLONG             "notlong.com"
#define METAMARK            "metamark.net"
#define PIURL               "piurl.com"
#define RI_MS               "ri.ms"
#define SN_IM               "sn.im"
#define SNIPR               "snipr.com"
#define SNIPURL             "snipurl.com"
#define SNURL               "snurl.com"
#define SRNK_NET            "srnk.net"
#define TINY_CC             "tiny.cc"
#define TINYURL             "tinyurl.com"
#define TR_IM               "tr.im"
#define TURL_CA             "turl.ca"
#define TWURL_NL            "twurl.nl"
#define UR_LY               "ur.ly"
#define URLOF               "urlof.com"
#define YIT_ME              "yit.me"
#define ZI_MA               "zi.ma"
#define ZZ_GD               "zz.gd"


/*******************/
/* Data Structures */
/*******************/

#define SURL surl_t

struct surl_struct;
struct surl_engine_struct;

typedef void (*surl_callback_t)(struct surl_struct *);

typedef struct surl_struct {
    char *options[SURLOPT_COUNT];
    surl_callback_t callback;
    char *services[SURL_MAX_SERVICES];
    struct surl_engine_struct *engines[SURL_MAX_SERVICES];
    int engine_cnt;
    pthread_t pthread;
    int threading;
    char *buffer;
    int buffer_size;
    int buffer_usage;
    int error;
    int debug;
} surl_t;

typedef void (*surl_shortener_t)(SURL *,void *);

/* an engine */
typedef struct surl_engine_struct {
    void *data; // miscellaneous data associated with this engine
    surl_shortener_t shortener; // the shortener function
} surl_engine_t;

/* an engine definition, used primarily by surl_engine_default_shortener() */
typedef struct {
    char *request_template; // template for URL+GET data
    char *post_template; // template for POST data we send
    char *response_template; // template for response
    int required[SURLOPT_COUNT]; // specification of required options
} surl_engine_def_t;


/******************/
/* surl functions */
/******************/

/* create a SURL with default and sensible settings */
SURL *surl_init();

/* add an engine to the SURL object */
int surl_register_engine(SURL *,char *,surl_engine_t *);

/* get the registered engine */
surl_engine_t *surl_get_engine(SURL *,char *);

/* get the error */
int surl_get_error(SURL *);

/* reset the error to none */
void surl_reset_error(SURL *);

/* turn debug on */
void surl_debug_on(SURL *);

/* turn debug off */
void surl_debug_off(SURL *);

/* is debug on? */
int surl_debug(SURL *);

/* set the value of a specified option */
void surl_setopt(SURL *,int,char *);

/* get the value of a specified option */
char *surl_getopt(SURL *,int);

/* sets the callback to be called when a process finished */
void surl_set_callback(SURL *,surl_callback_t);

/* gets the SURL callback */
surl_callback_t surl_get_callback(SURL *);

/* shortens the url */
int surl_shorten(SURL *);

/* waits for the given surl process to finish */
void surl_wait(SURL *);

/* cleans up a SURL, freeing any alocated memory */
void surl_cleanup(SURL *);


/************************/
/* Simple Use Functions */
/************************/

/* shortens the url with the specified service */
char *surl_simple_shorten(char *service,char *url);

/* shortens the url with the specified parameters */
char *surl_do_shorten(char *service,char *url,char *title,char *username,char *password,char *apikey);


/********************/
/* Engine functions */
/********************/

/* creates a blank engine */
surl_engine_t *surl_engine_init();

/* creates an engine from a url shortener function */
surl_engine_t *surl_engine_raw_create(surl_shortener_t);

/* creates an engine from an engine definition */
surl_engine_t *surl_engine_create(surl_engine_def_t *);

/* creates an engine definition */
surl_engine_def_t *surl_engine_def_create(char *,char *,char *);

/* registers a minimal GET engine with the SURL */
void surl_register_minimal_get_engine(SURL *,char *,char *);

/* registers a minimal POST engine with the SURL */
void surl_register_minimal_post_engine(SURL *,char *,char *,char *);

/* frees all memory used by the engine */
void surl_engine_cleanup(surl_engine_t *);


/******************/
/* cURL made easy */
/******************/

/* makes an http request */
char *easy_http_request(SURL *,char *url,char *postdata);


/*****************************/
/* General libsurl functions */
/*****************************/

/* return the libsurl version string */
char *libsurl_version();

/* the bugreport email address for libsurl */
char *libsurl_bugreport();

/* the configuration date of libsurl */
char *libsurl_config_date();

/* the build date of libsurl */
char *libsurl_build_date();

#endif // LIBSURL_FE_H
